﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace MarkusGrothoff.Windows.Forms {

    /// <summary>
    /// Stellen Buttons dar, die je nach Zustand verschiedene Icons anzeigen.
    /// </summary>
    public class MultiIconButton : Button {
        private Image disabledImage = null;
        private Image normalImage = null;
        private Image hoverImage = null;

        /// <summary>
        /// Ruft das Bild ab, das angezeigt wird, wenn der Button deaktiviert ist,
        /// oder legt es fest.
        /// </summary>
        public Image DisabledImage {
            get { return disabledImage; }
            set {
                if (!Equals(value, disabledImage)) {
                    disabledImage = value;
                    UpdateImage();
                }
            }
        }

        /// <summary>
        /// Ruft das Bild ab, das angezeigt wird, wenn der Button aktiviert ist,
        /// oder legt es fest.
        /// </summary>
        public Image NormalImage {
            get {
                return normalImage;
            }
            set {
                if (!Equals(value, normalImage)) {
                    normalImage = value;
                    UpdateImage();
                } else {
                }
            }
        }

        /// <summary>
        /// Ruft das Bild ab, das angezeigt wird, wenn der Cursor über dem Buttom schwebt,
        /// oder legt es fest.
        /// </summary>
        public Image HoverImage {
            get { return hoverImage; }
            set {
                if (!Equals(value, hoverImage)) {
                    hoverImage = value;
                    UpdateImage();
                }
            }
        }

        /// <summary>
        /// Aktualisiert das Bild je nach Status.
        /// </summary>
        private void UpdateImage() {
            if (!Enabled) {
                Image = disabledImage;
            } else {
                Point leftTop = ClientRectangle.Location;
                Point rightBottom = new Point(leftTop.X + ClientRectangle.Width, leftTop.Y + ClientRectangle.Height);
                if (PointToClient(MousePosition).X >= leftTop.X && PointToClient(MousePosition).X <= rightBottom.X
                    && PointToClient(MousePosition).Y >= leftTop.Y && PointToClient(MousePosition).Y <= rightBottom.Y) {
                    Image = hoverImage;
                } else {
                    Image = normalImage;
                }
            }
        }

        protected override void OnEnabledChanged(EventArgs e) {
            base.OnEnabledChanged(e);
            UpdateImage();
        }

        protected override void OnMouseEnter(EventArgs e) {
            base.OnMouseEnter(e);
            Image = hoverImage;
        }

        protected override void OnMouseLeave(EventArgs e) {
            base.OnMouseLeave(e);
            Image = normalImage;
        }
    }
}
