﻿using System;
using System.Drawing;
using System.Windows.Forms;
using System.IO;
using MarkusGrothoff.Media;

namespace Mp3Player {
    public partial class Mp3PlayerForm : Form {

        private const string AudioFileName = @"E:\VS 08 Projects\Mp3Player\rock.mp3";

        private int recentlyPlayedFileIndex = 0;

        public Mp3PlayerForm() {
            InitializeComponent();
            Size = Properties.Settings.Default.Mp3PlayerFormSize;
        }

        protected override void OnShown(System.EventArgs e) {
            base.OnShown(e);
            UpdatePlayList();
        }

        /// <summary>
        /// Aktualisiert die Playliste mit den Dateien und deren Länge.
        /// </summary>
        private void UpdatePlayList() {
            using (MediaPlayer o = new MediaPlayer()) {
                foreach (String p in Properties.Settings.Default.PlayList) {
                    ListViewItem newItem = new ListViewItem(p);
                    if (!File.Exists(p)) {
                        newItem.ForeColor = Color.Red;
                    } else {
                        try {
                            o.Open(p);
                            newItem.ForeColor = PlayListListView.ForeColor;
                            newItem.SubItems.Add(String.Format("{0:0}:{1:00}", o.Length.Minutes, o.Length.Seconds));
                        } catch (MediaException) {
                            newItem.ForeColor = Color.Red;
                        } finally {
                            o.Close();
                        }
                    }
                    PlayListListView.Items.Add(newItem);
                }
            }
        }

        /// <summary>
        /// Öffnet einen FileOpen-Dialog und fügt die ausgewählten Dateien der Playlist hinzu.
        /// </summary>
        private void FileOpen() {
            if (openFileDialog1.ShowDialog() == DialogResult.OK) {
                using (MediaPlayer o = new MediaPlayer()) {
                    foreach (String p in openFileDialog1.FileNames) {
                        ListViewItem newItem = new ListViewItem(p);
                        if (!File.Exists(p)) {
                            newItem.ForeColor = Color.Red;
                        } else {
                            try {
                                o.Open(p);
                                newItem.ForeColor = PlayListListView.ForeColor;
                                newItem.SubItems.Add(String.Format("{0:0}:{1:00}", o.Length.Minutes, o.Length.Seconds));
                            } catch (MediaException) {
                                newItem.ForeColor = Color.Red;
                            } finally {
                                o.Close();
                            }
                        }
                        PlayListListView.Items.Add(newItem);
                        Properties.Settings.Default.PlayList.Add(p);

                    }
                }
            }
        }

        /// <summary>
        /// Spielt die gewünschte Datei ab.
        /// </summary>
        /// <param name="fileName">Pfad- und Dateiname.</param>
        private void PlayFile(String fileName) {
            if (!File.Exists(fileName)) {
                MessageBox.Show("Die Datei konnte nicht geöffnet werden.", "Datei öffnen fehlgeschlagen",
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
            } else {
                try {                    
                    soundPlayerUserControl1.FileName = fileName;
                    soundPlayerUserControl1.Play();                    
                } catch (MediaException ex) {
                    MessageBox.Show(String.Format("Beim Öffnen der Datei ist folgender Fehler aufgetreten:{0}{1}",
                        Environment.NewLine + Environment.NewLine, ex.ToString()), "Datei öffnen fehlgeschlagen",
                         MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
        }

        private void PlayListListView_MouseDoubleClick(object sender, MouseEventArgs e) {
            // Ausgewählte Datei abspielen.
            ListViewItem item = ((ListView)sender).GetItemAt(e.X, e.Y);
            if (item != null) {
                PlayFile(item.Text);
                recentlyPlayedFileIndex = PlayListListView.Items.IndexOf(item);
            }
        }

        private void PlayListListView_KeyDown(object sender, KeyEventArgs e) {
            // Ausgewählte Dateien aus Playlist entfernen.
            if (e.KeyCode == Keys.Delete) {
                foreach (ListViewItem o in PlayListListView.SelectedItems) {
                    PlayListListView.Items.Remove(o);
                    Properties.Settings.Default.PlayList.Remove(o.Text);
                }
            }
        }

        private void FileOpenButton_Click(object sender, EventArgs e) {
            FileOpen();
        }

        private void Mp3PlayerForm_KeyDown(object sender, KeyEventArgs e) {
            if (e.Control && e.KeyCode == Keys.O) FileOpen();
        }

        private void Mp3PlayerForm_FormClosed(object sender, FormClosedEventArgs e) {
            Properties.Settings.Default.Mp3PlayerFormSize=Size ;
        }

        private void AboutButton_Click(object sender, EventArgs e) {
            using (AboutBox o = new AboutBox()) o.ShowDialog();
        }

        private void soundPlayerUserControl1_Played(object sender, EventArgs e) {
            //if (recentlyPlayedFileIndex + 1 < PlayListListView.Items.Count) PlayFile(PlayListListView.Items[recentlyPlayedFileIndex + 1].Text);
        }
    }
}
